@@ -4,6 +4,7 @@ import ftpclient,string,getpass,socket,os
 from ftpy_define import *
 from Tkinter import *
 import tkMessageBox
+import tkSimpleDialog
 
 ###############################################
 #	CUI mode class
@@ -222,7 +223,7 @@ class ftpcui:
 
 	#Output Help
 	def help(self):
-		self.ftpui.printHelp()
+		self.printHelp()
 
 	#Output prompt and Input command
 	def prompt(self):
@@ -289,19 +290,19 @@ class ftpgui(Frame):
 
 	#Print Message
 	def printMessage(self,color,msg):
-		self.textMessage.tag_config("c",foreground="red")
-		self.textMessage.insert(END,msg)
+	
+		self.textMessage.tag_config("red",foreground="red")
+		self.textMessage.tag_config("black",foreground="black")
+		self.textMessage.tag_config("blue",foreground="blue")
+		self.textMessage.insert(END,msg,color)
 		#Scroll message box
 		self.scrollMessage()
 	
 	#Print error massege
 	def printError(self,err_obj):
-		if len(err_obj.args) == 1:
-			args = err_obj.args[0]
-			self.textMessage.insert(END,args[4:])
-		elif len(err_obj.args) == 2:
-			args = err_obj.args[1]
-			self.textMessage.insert(END,args)
+		for e in err_obj:
+			self.printMessage(C_RED,e)
+		self.printMessage(C_BLACK,"\n")
 	#Scroll message box to bottom
 	def scrollMessage(self):
 		self.textMessage.yview(MOVETO,1)
@@ -338,14 +339,14 @@ class ftpgui(Frame):
 		self.printMessage(C_BLACK,"Check Hostname(" + hostname + ")..")
 		res = self.checkHostname(hostname)
 		if res <> C_OK:
-			self.printMessage(C_BLACK,hostname + " is not found\n")
+			self.printMessage(C_RED,hostname + " is not found\n")
 			return C_NG
-		self.printMessage(C_BLACK,"OK\n")
+		self.printMessage(C_BLUE,"OK\n")
 		#Connecting FTP server start
 		self.printMessage(C_BLACK,"Connect(" + hostname + ":" + portno + ")..")
 		res = self.ftpclient.connect()
 		if res == C_OK:
-			self.printMessage(C_BLACK,"OK\n")
+			self.printMessage(C_BLUE,"OK\n")
 			self.connectFlag = C_ON
 			#Set connection parameters
 			self.ftpclient.setUser(user)
@@ -357,23 +358,24 @@ class ftpgui(Frame):
 			res = self.ftpclient.login()
 			if res == C_OK:
 				self.getRemoteDirList()
-				self.printMessage(C_BLACK,"OK\n")
+				self.printMessage(C_BLUE,"OK\n")
 			else:
-				self.printMessage(C_BLACK,"Failure\n")
+				self.printMessage(C_RED,"Failure\n")
 		else:
-			self.printMessage(C_BLACK,"Failure\n")
+			self.printMessage(C_RED,"Failure\n")
 	#Disconnect to FTP server
 	def disconnect(self):
-		self.printMessage(C_BLACK,"Disconnect..")
-		res = self.ftpclient.close()
-		if res <> C_OK:
-			self.printMessage(C_BLACK,"Failure\n")
-			return C_NG
-		#clear to remote widget
-		self.listRemote.delete(0,END)
-		self.RemotePathVar.set("")
-		self.connectFlag = C_OFF
-		self.printMessage(C_BLACK,"OK\n")
+		if self.connectFlag == C_ON:
+			self.printMessage(C_BLACK,"Disconnect..")
+			res = self.ftpclient.close()
+			if res <> C_OK:
+				self.printMessage(C_RED,"Failure\n")
+				return C_NG
+			#clear to remote widget
+			self.listRemote.delete(0,END)
+			self.RemotePathVar.set("")
+			self.connectFlag = C_OFF
+			self.printMessage(C_BLUE,"OK\n")
 
 	#FTPy quit
 	def quit(self):
@@ -382,55 +384,57 @@ class ftpgui(Frame):
 
 	#Output version infomation
 	def showVersion(self):
-		tkMessageBox.showinfo(title="Version",message="FTPy\nVersion 0.1.0\nCreater:shinkichi")
+		tkMessageBox.showinfo(title="Version",message="FTPy\nVersion 0.1.1\nCreater:shinkichi")
 	
 	#File update(Local -> Remote)
 	def update(self):
 		#Get select file
-		indexed = self.listLocal.curselection()
-		if indexed.__len__() == 0:
+		indexes = self.listLocal.curselection()
+		if len(indexes) == 0:
 			return C_NG
-		item = self.listLocal.get(indexed[0])
-		#Check item
-		if item[0] == "-":	#File
-			filename = item[string.find(item," ")+1:]
-			self.printMessage(C_BLACK,"Update(" + filename + ")..")
-			#Call putLocalFile method
-			res = self.ftpclient.putLocalFile(filename,filename)
-			if res == C_OK:
-				self.printMessage(C_BLACK,"OK\n")
-				self.getRemoteDirList()
-			else:
-				self.printMessage(C_BLACK,"Failure\n")
+		for e in indexes:
+			item = self.listLocal.get(e)
+			#Check item
+			if item[0] == "-":	#File
+				filename = item[string.rfind(item," ")+1:]
+				self.printMessage(C_BLACK,"Update(" + filename + ")..")
+				#Call putLocalFile method
+				res = self.ftpclient.putLocalFile(filename,filename)
+				if res == C_OK:
+					self.printMessage(C_BLUE,"OK\n")
+					self.getRemoteDirList()
+				else:
+					self.printMessage(C_RED,"Failure\n")
 	
 	#File download(Remote -> Local)
 	def download(self):
 		#Get select file
-		indexed = self.listRemote.curselection()
-		if indexed.__len__() == 0:
+		indexes = self.listRemote.curselection()
+		if len(indexes) == 0:
 			return C_NG
-		item = self.listRemote.get(indexed[0])
-		#Check item
-		if item[0] == "-":	#File
-			filename = item[string.find(item," ")+1:]
-			self.printMessage(C_BLACK,"Download(" + filename + ")..")
-			#Call getRemoteFile method
-			res = self.ftpclient.getRemoteFile(filename,filename)
-			if res == C_OK:
-				self.printMessage(C_BLACK,"OK\n")
-				self.getLocalDirList()
-			else:
-				self.printMessage(C_BLACK,"Failure\n")
-		elif item[0] == "d":	#Directory
-			dirname = item[string.find(item," ")+1:]
-			self.printMessage(C_BLACK,"Download(" + dirname + ")..")
-			#Call getRemoteDir method
-			res = self.ftpclient.getRemoteDir(dirname)
-			if res == C_OK:
-				self.printMessage(C_BLACK,"OK\n")
-				self.getLocalDirList()
-			else:
-				self.printMessage(C_BLACK,"Failure\n")
+		for e in indexes:
+			item = self.listRemote.get(e)
+			#Check item
+			if item[0] == "-":	#File
+				filename = item[string.rfind(item," ")+1:]
+				self.printMessage(C_BLACK,"Download(" + filename + ")..")
+				#Call getRemoteFile method
+				res = self.ftpclient.getRemoteFile(filename,filename)
+				if res == C_OK:
+					self.printMessage(C_BLUE,"OK\n")
+					self.getLocalDirList()
+				else:
+					self.printMessage(C_RED,"Failure\n")
+			elif item[0] == "d":	#Directory
+				dirname = item[string.rfind(item," ")+1:]
+				self.printMessage(C_BLACK,"Download(" + dirname + ")..")
+				#Call getRemoteDir method
+				res = self.ftpclient.getRemoteDir(dirname)
+				if res == C_OK:
+					self.printMessage(C_BLUE,"OK\n")
+					self.getLocalDirList()
+				else:
+					self.printMessage(C_RED,"Failure\n")
 
 	#Refresh dirctory list(Local,Remote)
 	def refresh(self):
@@ -452,7 +456,7 @@ class ftpgui(Frame):
 		#Call getLocalDirData method
 		res = self.ftpclient.getLocalDirData(current_dir[0])
 		if res <> C_OK:
-			self.printMessage(C_BLACK,"Connection..Failure\n")
+			self.printMessage(C_RED,"Connection..Failure\n")
 			return C_NG
 		#Get local directory data
 		dir_obj = []
@@ -461,15 +465,17 @@ class ftpgui(Frame):
 		#Insert local directory entry
 		self.listLocal.insert(END,"d ..")
 		for e in dir_obj:
+			filesize = str(os.path.getsize(e))
+			filesize = " " * (11 - len(filesize)) + str(filesize)
 			if e[0] <> ".":
 				if os.path.isfile(e):
-					item = "- " + e
+					item = "- " + filesize + " " + e
 				elif os.path.isdir(e):
-					item = "d " + e
+					item = "d " + filesize + " " + e
 				elif os.path.islink(e):
-					item = "l " + e
+					item = "l " + filesize + " " + e
 				else:
-					item = "? " + e
+					item = "? " + filesize + " " + e
 				self.listLocal.insert(END,item)
 	
 	#Refresh remote directory list
@@ -477,7 +483,7 @@ class ftpgui(Frame):
 		#Get remote current directory
 		res = self.ftpclient.getCurrentRemoteDir()
 		if res <> C_OK:
-			self.printMessage(C_BLACK,"Connection..Failure\n")
+			self.printMessage(C_RED,"Connection..Failure\n")
 			return C_NG
 		current_dir = self.ftpclient.getMsg_Obj()
 		self.RemotePathVar.set(current_dir[0])
@@ -486,7 +492,7 @@ class ftpgui(Frame):
 		#Get remote directory data 
 		res = self.ftpclient.getRemoteDirData()
 		if res <> C_OK:
-			self.printMessage(C_BLACK,"Connection..Failure\n")
+			self.printMessage(C_RED,"Connection..Failure\n")
 			return C_NG
 		dir_obj = []
 		dir_obj = self.ftpclient.getRemoteDir_Obj()
@@ -494,51 +500,100 @@ class ftpgui(Frame):
 		if current_dir[0] <> "/":
 			self.listRemote.insert(END,"d ..")
 		for e in dir_obj:
-			obj_attribute = e[0]
-			obj_name = e[string.rfind(e," ")+1:]
-			item = obj_attribute + " " + obj_name
-			self.listRemote.insert(END,item)
+			if e[0] == "d" or e[0] == "-" or e[0] == "l":
+				line_ent = []
+				line_ent = self.splitDirEntry(e)
+				permission = line_ent[0] 
+				filesize = line_ent[4]
+				filename = line_ent[len(line_ent)-1]
+				if len(filesize) < 10:
+					add_space = " " * (11 - len(filesize))
+					filesize = add_space + filesize
+				item = permission + filesize + " " + filename 
+				self.listRemote.insert(END,item)
+
+	#Split directory entry
+	def splitDirEntry(self,entry):
+		line_ent = []
+		for e in string.split(entry," "):
+			if e <> "":
+				line_ent.append(e)
+		return line_ent
+
+	#Change mode(Remote)
+	def changeMode(self):
+		indexed = self.listRemote.curselection()
+		if len(indexed) == 0:
+			return C_NG
+		item = self.listRemote.get(indexed[0])
+		#Check item
+		if item[0] == "-" or item[0] == "d":	#File,Dirctory
+			objname = item[string.rfind(item," ")+1:]
+		mode = tkSimpleDialog.askstring("Change Mode","Mode\nexampl:744 +x")
+		if mode <> None:
+			self.printMessage(C_BLACK,"chmod(" + mode + ")..")
+			res = self.ftpclient.changeModeRemoteFile(objname,mode)
+			if res == C_OK:
+				self.printMessage(C_BLUE,"OK\n")
+				self.getRemoteDirList()
+			else:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+
+	#Create directory(Remote)
+	def createDirectory(self):
+		dirname = tkSimpleDialog.askstring("Create Directory","Directory Name")
+		if dirname <> None:
+			self.printMessage(C_BLACK,"mkdir(" + dirname + ")..")
+			res = self.ftpclient.createRemote_dir(dirname)
+			if res == C_OK:
+				self.printMessage(C_BLUE,"OK\n")
+				self.getRemoteDirList()
+			else:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+				
 
 	#Remote directory list(Event)
 	def doubleClickRemote(self,event):
 		#Get select item
 		indexed = self.listRemote.curselection()
-		if indexed.__len__() == 0:
+		if len(indexed) == 0:
 			return C_NG
 		item = self.listRemote.get(indexed[0])
 		#Check item
 		if item[0] == "d":	#Directory
-			dirname = item[string.find(item," ")+1:]
+			dirname = item[string.rfind(item," ")+1:]
 			self.printMessage(C_BLACK,"cd(" + dirname + ")..")
 			#Change remote directory
 			res = self.ftpclient.changeRemoteDir(dirname)
 			if res <> C_OK:
-				self.printMessage(C_BLACK,"Connection..Failure\n")
+				self.printMessage(C_RED,"Connection..Failure\n")
 				return C_NG
-			self.printMessage(C_BLACK,"OK\n")
+			self.printMessage(C_BLUE,"OK\n")
 			self.getRemoteDirList()
 
 	#Local directory list(Event)
 	def doubleClickLocal(self,event):
 		#Get select item
 		indexed = self.listLocal.curselection()
-		if indexed.__len__() == 0:
+		if len(indexed) == 0:
 			return C_NG
 		item = self.listLocal.get(indexed[0])
 		#Check item
 		if item[0] == "d":	#Directory
-			dirname = item[string.find(item," ")+1:]
+			dirname = item[string.rfind(item," ")+1:]
 			#Change local directory
 			res = self.ftpclient.changeLocalDir(dirname)
 			if res <> C_OK:
-				self.printMessage(C_BLACK,"Connection..Failure\n")
+				self.printMessage(C_RED,"Connection..Failure\n")
 				return C_NG
 			self.getLocalDirList()
 
 	#Print FTPy widget
 	def printGUI(self):
 		### Tk ###
-		self.master.title("FTPy-0.1.0")
+		self.master.title("FTPy-0.1.1")
 		
 		### Frame ###
 		frameTop = Frame(self)
@@ -576,6 +631,12 @@ class ftpgui(Frame):
 		#Version
 		menuHelp.add_command(label="version",command=self.showVersion)
 		self.master.config(menu=menuTop)
+		## Popup menu(Remote) ##
+		self.menuPopupRemote = Menu()
+		#Change mode
+		self.menuPopupRemote.add_command(label="Change mode",command=self.changeMode)
+		#Create directory
+		self.menuPopupRemote.add_command(label="Create directory",command=self.createDirectory)
 
 		### Host infomation ###
 		#Hostname
@@ -613,9 +674,9 @@ class ftpgui(Frame):
 
 		### Toolbar ###
 		#Toolbar connect
-		btnConnect = Button(frameToolbar,bitmap="info",width="20",height="20",command=self.connect)
+		btnConnect = Button(frameToolbar,bitmap="@connect.xbm",width="24",height="24",command=self.connect)
 		#Toolbar disconnect
-		btnDisconnect = Button(frameToolbar,bitmap="error",width="20",height="20",command=self.disconnect)
+		btnDisconnect = Button(frameToolbar,bitmap="error",width="24",height="24",command=self.disconnect)
 		btnConnect.grid(row=0,column=0,sticky=W+E)
 		btnDisconnect.grid(row=0,column=1,sticky=W)
 		
@@ -623,10 +684,10 @@ class ftpgui(Frame):
 		#Local Path
 		self.LocalPathVar = StringVar()
 		self.LocalPathVar.set("")
-		entryLocalPath = Entry(frameLocal,width="30",textvariable=self.LocalPathVar)
+		entryLocalPath = Entry(frameLocal,width="34",textvariable=self.LocalPathVar)
 		entryLocalPath.grid(row=0,column=0)
 		#List
-		self.listLocal = Listbox(frameLocal,selectmode="single",width=30,height=10,bg="white")
+		self.listLocal = Listbox(frameLocal,selectmode=EXTENDED,width=40,height=10,bg="white",font="system")
 		scrvLocal = Scrollbar(frameLocal,orient=VERTICAL,command=self.listLocal.yview)
 		scrhLocal = Scrollbar(frameLocal,orient=HORIZONTAL,command=self.listLocal.xview)
 		self.listLocal.config(xscrollcommand=scrhLocal.set,yscrollcommand=scrvLocal.set)
@@ -646,20 +707,21 @@ class ftpgui(Frame):
 		#Remote Path
 		self.RemotePathVar = StringVar()
 		self.RemotePathVar.set("")
-		entryRemotePath = Entry(frameRemote,width="30",textvariable=self.RemotePathVar)
+		entryRemotePath = Entry(frameRemote,width="34",textvariable=self.RemotePathVar)
 		entryRemotePath.grid(row=0,column=0)
 		#List
-		self.listRemote = Listbox(frameRemote,selectmode="single",width=30,height=10,bg="white")
+		self.listRemote = Listbox(frameRemote,selectmode=EXTENDED,width=40,height=10,bg="white",font="system")
 		scrvRemote = Scrollbar(frameRemote,orient=VERTICAL,command=self.listRemote.yview)
 		scrhRemote = Scrollbar(frameRemote,orient=HORIZONTAL,command=self.listRemote.xview)
 		self.listRemote.config(xscrollcommand=scrhRemote.set,yscrollcommand=scrvRemote.set)
 		self.listRemote.bind("<Double-Button-1>",self.doubleClickRemote)
+		self.listRemote.bind("<ButtonRelease-3>",self.button3ClickRemote)
 		self.listRemote.grid(row=1,column=0)
 		scrvRemote.grid(row=1,column=1,sticky=NS)
 		scrhRemote.grid(row=2,column=0,sticky=EW)
 
 		### Message ###
-		self.textMessage = Text(frameMessage,width=70,height=6)
+		self.textMessage = Text(frameMessage,width=78,height=6)
 		scrvMessage = Scrollbar(frameMessage,orient=VERTICAL,command=self.textMessage.yview)
 		self.textMessage.config(yscrollcommand=scrvMessage.set)
 		self.textMessage.grid(row=0,column=0)
@@ -669,6 +731,10 @@ class ftpgui(Frame):
 	def pressEnter(self,event):
 		self.connect()
 
+	#Button3 Click(Remote)
+	def button3ClickRemote(self,event):
+		self.menuPopupRemote.tk_popup(event.x_root,event.y_root)
+
 	#Class initial
 	def __init__(self,hostname="",portno=21,pasv=C_OFF):
 		master=None
@@ -683,3 +749,5 @@ class ftpgui(Frame):
 		if pasv == C_ON:
 			self.ftpclient.setPasvmode()
 
+	
+	
